from typing import Any, Dict
from django import http
from django.http import HttpRequest, HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse, reverse_lazy
from django.utils import translation
from django.views import generic
from web.forms import ProductForm

from web import models


class ProductCreateView(generic.CreateView):
    model = models.Product
    template_name = 'product/create_update.html'
    form_class = ProductForm
    extra_context = {
        'button_text': translation.gettext('Create')
    }
    
    def dispatch(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        print(translation.gettext('Create'))
        return super().dispatch(request, *args, **kwargs)

    def get_success_url(self) -> str:
        return reverse('product_list')    
 

class ProductUpdateView(generic.UpdateView):
    model = models.Product
    template_name = 'product/create_update.html'
    form_class = ProductForm
    pk_url_kwarg = 'id'
    context_object_name = 'product'
    extra_context = {
        'button_text': 'Update'
    }
    

class ProductDetailView(generic.DetailView):
    model = models.Product
    template_name = 'product/detail.html'
    pk_url_kwarg = 'id'
    context_object_name = 'product'


class ProductListView(generic.ListView):
    model = models.Product
    template_name = 'product/list.html'
    context_object_name = 'products'
    paginate_by = 3


class ProductDeleteView(generic.DeleteView):
    model = models.Product
    context_object_name = 'products'
    pk_url_kwarg = 'id'


class AddProductToBucketView(generic.CreateView):
    model = models.Bucket
    fields = ['product']
    template_name = 'bucket/create.html'
    extra_context = {
        'button_text': 'Add'
    }
    success_url = reverse_lazy('product_list')
     
    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse:
        print(request.POST)
        self.product = get_object_or_404(models.Product, id=request.POST.get('product'))
        form = self.get_form()
        
        if form.is_valid():
            product_in_bucket = self.model.objects.filter(product_id=self.product.id).first()
            
            if self.product.qty != 0:
                if not product_in_bucket:
                    form.instance.qty = 1
                    form.save()
                        
                elif product_in_bucket:
                    product_in_bucket.qty += 1
                    product_in_bucket.save(update_fields=['qty'])
                    
                self.product.qty -= 1
                self.product.save()

            return redirect(self.success_url)
        
        return self.form_invalid(form)


class BucketListView(generic.ListView):
    model = models.Bucket
    template_name = 'bucket/detail.html'
    context_object_name = 'bucket'
    
    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        total_sum = 0
        qs = []

        for bucket_obj in self.model.objects.select_related('product'):
            obj = vars(bucket_obj)
            total = bucket_obj.product.price * bucket_obj.qty
            obj['total'] = total
            obj['product'] = bucket_obj.product
            qs.append(obj)
            total_sum += total

        kwargs['qs'] = qs
        return super().get_context_data(total=total_sum, **kwargs)
